---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2013).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity my_ring_counter is
	generic (N: INTEGER:= 4);
	port (clock, astart: in std_logic;
			E: in std_logic;
	      Q: out std_logic_vector (N-1 downto 0));
end my_ring_counter;

architecture Behavioral of my_ring_counter is

	component dffe is
		 Port ( d : in  STD_LOGIC;
				  clrn: in std_logic:= '1';
				  prn: in std_logic:= '1';
				  clk : in  STD_LOGIC;
				  ena: in std_logic;
				  q : out  STD_LOGIC);
	end component;

	signal Qt: std_logic_vector (N-1 downto 0);
	
begin

	fa: dffe port map (d => Qt(N-1), clrn => '1', prn => astart, clk => clock, ena => E, q => Qt(0));
	
	gi: for i in 1 to N-1 generate
				fi: dffe port map (d => Qt(i-1), clrn => astart, prn => '1', clk => clock, ena => E, q => Qt(i));
		 end generate;
	Q <= Qt;
end Behavioral;

